--------------------------------------------------------------------------------
-- CERN BE-CO-HT
-- Mock Turtle
-- https://gitlab.cern.ch/coht/mockturtle
--------------------------------------------------------------------------------
--
-- unit name:   svec_mt_demo
--
-- description: Top entity for Mock Turtle SVEC demo.
--
-- This demo instantiates a Mock Turtle CPU with two cores, each connected to
-- a WB GPIO peripheral. Reading and writing to the GPIOs controls the LEDs
-- and LEMO I/O on the front panel of the SVEC.
--
--------------------------------------------------------------------------------
-- Copyright (c) 2014-2019 CERN (home.cern)
--------------------------------------------------------------------------------
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 2.0 (the "License"); you may not use this file except
-- in compliance with the License. You may obtain a copy of the License at
-- http://solderpad.org/licenses/SHL-2.0.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.gencores_pkg.all;
use work.wishbone_pkg.all;
use work.vme64x_pkg.all;
use work.mock_turtle_pkg.all;
use work.mt_mqueue_pkg.all;
use work.synthesis_descriptor.all;
use work.wr_fabric_pkg.all;
use work.wr_svec_pkg.all;
use work.wr_board_pkg.all;

library unisim;
use unisim.vcomponents.all;

entity svec_mt_demo_wr is
  generic (
    -- Bypass VME core, useful only in simulation
    g_SIM_BYPASS_VME : boolean := false;
    -- Enables simulation mode (reduces timeouts)
    g_SIMULATION     : boolean := false);
  port (
    -- resets and clocks
    rst_a_n_i           : in std_logic;
    clk_20m_vcxo_i      : in std_logic;  -- 20MHz VCXO clock
    clk_125m_pllref_p_i : in std_logic;
    clk_125m_pllref_n_i : in std_logic;
    clk_125m_gtp_n_i    : in std_logic;  -- 125 MHz GTP reference
    clk_125m_gtp_p_i    : in std_logic;

    ---------------------------------------------------------------------------
    -- SPI interfaces to DACs
    ---------------------------------------------------------------------------

    pll20dac_din_o    : out std_logic;
    pll20dac_sclk_o   : out std_logic;
    pll20dac_sync_n_o : out std_logic;
    pll25dac_din_o    : out std_logic;
    pll25dac_sclk_o   : out std_logic;
    pll25dac_sync_n_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SFP I/O for transceiver
    ---------------------------------------------------------------------------

    sfp_txp_o         : out   std_logic;
    sfp_txn_o         : out   std_logic;
    sfp_rxp_i         : in    std_logic;
    sfp_rxn_i         : in    std_logic;
    sfp_mod_def0_i    : in    std_logic;  -- sfp detect
    sfp_mod_def1_b    : inout std_logic;  -- scl
    sfp_mod_def2_b    : inout std_logic;  -- sda
    sfp_rate_select_o : out   std_logic;
    sfp_tx_fault_i    : in    std_logic;
    sfp_tx_disable_o  : out   std_logic;
    sfp_los_i         : in    std_logic;

    ---------------------------------------------------------------------------
    -- Carrier I2C EEPROM
    ---------------------------------------------------------------------------

    carrier_scl_b : inout std_logic;
    carrier_sda_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- Onewire interface
    ---------------------------------------------------------------------------

    onewire_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- UART
    ---------------------------------------------------------------------------

    uart_rxd_i : in  std_logic;
    uart_txd_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SPI (flash is connected to SFPGA and routed to AFPGA
    -- once the boot process is complete)
    ---------------------------------------------------------------------------

    spi_sclk_o : out std_logic;
    spi_ncs_o  : out std_logic;
    spi_mosi_o : out std_logic;
    spi_miso_i : in  std_logic;

    -- SVEC Front panel LEDs and GPIO
    fp_led_line_oen_o : out   std_logic_vector(1 downto 0);
    fp_led_line_o     : out   std_logic_vector(1 downto 0);
    fp_led_column_o   : out   std_logic_vector(3 downto 0);
    fp_gpio1_a2b_o    : out   std_logic;
    fp_gpio2_a2b_o    : out   std_logic;
    fp_gpio34_a2b_o   : out   std_logic;
    fp_gpio_b         : inout std_logic_vector(3 downto 0);
    -- Bypass VME core, useful only in simulation
    -- synthesis translate_off
    sim_wb_i          : in    t_wishbone_slave_in := cc_dummy_slave_in;
    sim_wb_o          : out   t_wishbone_slave_out;
    -- synthesis translate_on
    -- VME Interface pins
    VME_AS_n_i        : in    std_logic;
    VME_RST_n_i       : in    std_logic;
    VME_WRITE_n_i     : in    std_logic;
    VME_AM_i          : in    std_logic_vector(5 downto 0);
    VME_DS_n_i        : in    std_logic_vector(1 downto 0);
    VME_GA_i          : in    std_logic_vector(5 downto 0);
    VME_BERR_b        : inout std_logic;
    VME_DTACK_n_b     : inout std_logic;
    VME_RETRY_n_o     : out   std_logic;
    VME_RETRY_OE_o    : out   std_logic;
    VME_LWORD_n_b     : inout std_logic;
    VME_ADDR_b        : inout std_logic_vector(31 downto 1);
    VME_DATA_b        : inout std_logic_vector(31 downto 0);
    VME_BBSY_n_i      : in    std_logic;
    VME_IRQ_n_o       : out   std_logic_vector(6 downto 0);
    VME_IACK_n_i      : in    std_logic;
    VME_IACKIN_n_i    : in    std_logic;
    VME_IACKOUT_n_o   : out   std_logic;
    VME_DTACK_OE_b    : inout std_logic;
    VME_DATA_DIR_b    : inout std_logic;
    VME_DATA_OE_N_b   : inout std_logic;
    VME_ADDR_DIR_b    : inout std_logic;
    VME_ADDR_OE_N_b   : inout std_logic);
end svec_mt_demo_wr;

architecture arch of svec_mt_demo_wr is

  -- Host & Remote Message Queues configuration

  constant c_MT_CONFIG : t_mt_config := (
    app_id                                                    => x"d331d331",
    cpu_count                                                 => 2,
    cpu_config                                                => (others =>
                   (memsize                                   => 8192,
                    hmq_config                                => (2,
                                   (0                         => (entries_bits => 2,
                                          width_bits          => 7,
                                          header_bits         => 2,
                                          endpoint_id         => x"0000_0000",
                                          enable_config_space => false),
                                    1                         => (entries_bits => 5,
                                          width_bits          => 4,
                                          header_bits         => 3,
                                          endpoint_id         => x"0000_0000",
                                          enable_config_space => false),
                                    others                    => (c_DUMMY_MT_MQUEUE_SLOT))),
                    rmq_config                                => (1,
                                   (0                         => (entries_bits => 2,
                                          width_bits          => 7,
                                          header_bits         => 6,
                                          endpoint_id         => x"0000_0000",
                                          enable_config_space => true),
                                    others                    => (c_DUMMY_MT_MQUEUE_SLOT))))),
    shared_mem_size                                           => 2048);


  function f_to_integer(x : boolean) return integer is
  begin
    if (x) then
      return 1;
    else
      return 0;
    end if;
  end f_to_integer;


  constant c_NUM_WB_MASTERS : integer := 3;
  constant c_NUM_WB_SLAVES  : integer := 1;

  constant c_MASTER_VME : integer := 0;

  constant c_SLAVE_VIC     : integer := 0;
  constant c_SLAVE_MT      : integer := 1;
  constant c_SLAVE_WR_CORE : integer := 2;

  constant c_DESC_SYNTHESIS : integer := 3;
  constant c_DESC_REPO_URL  : integer := 4;

  constant c_WRC_BRIDGE_SDB : t_sdb_bridge :=
    f_xwb_bridge_manual_sdb(x"0003ffff", x"00030000");

  constant c_INTERCONNECT_LAYOUT : t_sdb_record_array(c_NUM_WB_MASTERS + 1 downto 0) := (
    c_SLAVE_VIC      => f_sdb_embed_device(c_XWB_VIC_SDB, x"00002000"),
    c_SLAVE_MT       => f_sdb_embed_device(c_MOCK_TURTLE_SDB, x"00020000"),
    c_SLAVE_WR_CORE  => f_sdb_embed_bridge(c_WRC_BRIDGE_SDB, x"00040000"),
    c_DESC_SYNTHESIS => f_sdb_embed_synthesis(c_SDB_SYNTHESIS_INFO),
    c_DESC_REPO_URL  => f_sdb_embed_repo_url(c_SDB_REPO_URL));

  constant c_SDB_ADDRESS : t_wishbone_address := x"00000000";

  constant c_VIC_VECTOR_TABLE : t_wishbone_address_array(0 to 1) := (
    0 => x"00020000",                   -- MT Mqueue interrupt
    1 => x"00020001");                  -- MT Debug Msg interrupt

  signal cnx_master_out : t_wishbone_master_out_array(c_NUM_WB_MASTERS-1 downto 0);
  signal cnx_master_in  : t_wishbone_master_in_array(c_NUM_WB_MASTERS-1 downto 0);

  signal cnx_slave_out : t_wishbone_slave_out_array(c_NUM_WB_SLAVES-1 downto 0);
  signal cnx_slave_in  : t_wishbone_slave_in_array(c_NUM_WB_SLAVES-1 downto 0);

  signal clk_sys : std_logic;

  signal rst_n_sys, rst_sys : std_logic;

  signal dp_wb_out : t_wishbone_master_out_array(1 downto 0);
  signal dp_wb_in  : t_wishbone_master_in_array(1 downto 0);

  signal cpu0_gpio_oen, cpu1_gpio_oen : std_logic_vector(23 downto 0);
  signal cpu0_gpio_out, cpu1_gpio_out : std_logic_vector(23 downto 0);
  signal cpu_gpio_oen                 : std_logic_vector(23 downto 0);
  signal cpu_gpio_out, cpu_gpio_in    : std_logic_vector(23 downto 0);

  signal VME_DATA_b_out    : std_logic_vector(31 downto 0);
  signal VME_ADDR_b_out    : std_logic_vector(31 downto 1);
  signal VME_LWORD_n_b_out : std_logic;
  signal VME_DATA_DIR_int  : std_logic;
  signal VME_ADDR_DIR_int  : std_logic;
  signal VME_BERR_n        : std_logic;
  signal VME_IRQ_n         : std_logic_vector(6 downto 0);

  signal mt_hmq_in_irq  : std_logic;
  signal mt_hmq_out_irq : std_logic;
  signal mt_console_irq : std_logic;
  signal mt_notify_irq  : std_logic;
  signal vic_master_irq : std_logic;

  signal fp_gpio_in : std_logic_vector(3 downto 0);

-- clock and reset
  signal areset_n : std_logic;

-- I2C EEPROM
  signal eeprom_sda_in  : std_logic;
  signal eeprom_sda_out : std_logic;
  signal eeprom_scl_in  : std_logic;
  signal eeprom_scl_out : std_logic;

-- SFP
  signal sfp_sda_in  : std_logic;
  signal sfp_sda_out : std_logic;
  signal sfp_scl_in  : std_logic;
  signal sfp_scl_out : std_logic;

-- OneWire
  signal onewire_data : std_logic;
  signal onewire_oe   : std_logic;

-- LEDs and GPIO
  signal pps         : std_logic;
  signal pps_led     : std_logic;
  signal pps_ext_in  : std_logic;
  signal svec_led    : std_logic_vector(15 downto 0);
  signal wr_led_link : std_logic;
  signal wr_led_act  : std_logic;

  signal wrc_src_out : t_wrf_source_out;
  signal wrc_src_in  : t_wrf_source_in;
  signal wrc_snk_out : t_wrf_sink_out;
  signal wrc_snk_in  : t_wrf_sink_in;

  attribute keep               : string;
  attribute keep of fp_gpio_in : signal is "true";

  signal mt2ep : t_mt_rmq_endpoint_iface_out;
  signal ep2mt : t_mt_rmq_endpoint_iface_in;

begin  -- architecture arch
  areset_n <= VME_RST_n_i and rst_a_n_i;

  gen_with_vme64_core : if not g_SIM_BYPASS_VME generate
    U_VME_Core : xvme64x_core
      generic map (
        g_CLOCK_PERIOD    => 16,
        g_DECODE_AM       => true,
        g_USER_CSR_EXT    => false,
        g_WB_GRANULARITY  => BYTE,
        g_MANUFACTURER_ID => c_CERN_ID,
        g_BOARD_ID        => c_SVEC_ID,
        g_REVISION_ID     => c_SVEC_REVISION_ID,
        g_PROGRAM_ID      => c_SVEC_PROGRAM_ID)
      port map (
        clk_i           => clk_sys,
        rst_n_i         => rst_n_sys,
        vme_i.as_n      => VME_AS_n_i,
        vme_i.rst_n     => rst_n_sys,
        vme_i.write_n   => VME_WRITE_n_i,
        vme_i.am        => VME_AM_i,
        vme_i.ds_n      => VME_DS_n_i,
        vme_i.ga        => VME_GA_i,
        vme_i.lword_n   => VME_LWORD_n_b,
        vme_i.addr      => VME_ADDR_b,
        vme_i.data      => VME_DATA_b,
        vme_i.iack_n    => VME_IACK_n_i,
        vme_i.iackin_n  => VME_IACKIN_n_i,
        vme_o.berr_n    => VME_BERR_n,
        vme_o.dtack_n   => VME_DTACK_n_b,
        vme_o.retry_n   => VME_RETRY_n_o,
        vme_o.retry_oe  => VME_RETRY_OE_o,
        vme_o.lword_n   => VME_LWORD_n_b_out,
        vme_o.data      => VME_DATA_b_out,
        vme_o.addr      => VME_ADDR_b_out,
        vme_o.irq_n     => VME_IRQ_n,
        vme_o.iackout_n => VME_IACKOUT_n_o,
        vme_o.dtack_oe  => VME_DTACK_OE_b,
        vme_o.data_dir  => VME_DATA_DIR_int,
        vme_o.data_oe_n => VME_DATA_OE_N_b,
        vme_o.addr_dir  => VME_ADDR_DIR_int,
        vme_o.addr_oe_n => VME_ADDR_OE_N_b,
        wb_o            => cnx_slave_in(c_MASTER_VME),
        wb_i            => cnx_slave_out(c_MASTER_VME),
        int_i           => vic_master_irq);

    VME_DATA_b    <= VME_DATA_b_out    when VME_DATA_DIR_int = '1' else (others => 'Z');
    VME_ADDR_b    <= VME_ADDR_b_out    when VME_ADDR_DIR_int = '1' else (others => 'Z');
    VME_LWORD_n_b <= VME_LWORD_n_b_out when VME_ADDR_DIR_int = '1' else 'Z';

    VME_ADDR_DIR_b <= VME_ADDR_DIR_int;
    VME_DATA_DIR_b <= VME_DATA_DIR_int;

    --  BERR and IRQ vme signals are inverted by the drivers. See SVEC schematics.
    VME_BERR_b  <= not VME_BERR_n;
    VME_IRQ_n_o <= not VME_IRQ_n;
  end generate gen_with_vme64_core;

  gen_without_vme64_core : if g_SIM_BYPASS_VME generate
    -- synthesis translate_off
    cnx_slave_in(c_MASTER_VME) <= sim_wb_i;
    sim_wb_o                   <= cnx_slave_out(c_MASTER_VME);
  -- synthesis translate_on
  end generate gen_without_vme64_core;

  U_Intercon : xwb_sdb_crossbar
    generic map (
      g_NUM_MASTERS => c_NUM_WB_SLAVES,
      g_NUM_SLAVES  => c_NUM_WB_MASTERS,
      g_REGISTERED  => true,
      g_WRAPAROUND  => true,
      g_LAYOUT      => c_INTERCONNECT_LAYOUT,
      g_SDB_ADDR    => c_SDB_ADDRESS)
    port map (
      clk_sys_i => clk_sys,
      rst_n_i   => rst_n_sys,
      slave_i   => cnx_slave_in,
      slave_o   => cnx_slave_out,
      master_i  => cnx_master_in,
      master_o  => cnx_master_out);

  U_VIC : xwb_vic
    generic map (
      g_INTERFACE_MODE      => PIPELINED,
      g_ADDRESS_GRANULARITY => BYTE,
      g_NUM_INTERRUPTS      => 4,
      g_INIT_VECTORS        => c_VIC_VECTOR_TABLE)
    port map (
      clk_sys_i    => clk_sys,
      rst_n_i      => rst_n_sys,
      slave_i      => cnx_master_out(c_SLAVE_VIC),
      slave_o      => cnx_master_in(c_SLAVE_VIC),
      irqs_i(0)    => mt_hmq_in_irq,
      irqs_i(1)    => mt_hmq_out_irq,
      irqs_i(2)    => mt_console_irq,
      irqs_i(3)    => mt_notify_irq,
      irq_master_o => vic_master_irq);

  U_Mock_Turtle : mock_turtle_core
    generic map (
      g_CONFIG                     => c_MT_CONFIG,
      g_WITH_WHITE_RABBIT          => true)
    port map (
      clk_i        => clk_sys,
      rst_n_i      => rst_n_sys,
      dp_master_o  => dp_wb_out,
      dp_master_i  => dp_wb_in,
      host_slave_i => cnx_master_out(c_SLAVE_MT),
      host_slave_o => cnx_master_in(c_SLAVE_MT),



      hmq_in_irq_o  => mt_hmq_in_irq,
      hmq_out_irq_o => mt_hmq_out_irq,
      notify_irq_o  => mt_notify_irq,
      console_irq_o => mt_console_irq);


  U_Ethernet_Endpoint: entity work.mt_rmq_ethernet_endpoint
    generic map (
      g_CONFIG => c_MT_CONFIG)
    port map (
      clk_i     => clk_sys,
      rst_n_i   => rst_n_sys,

      mt_rmq_i  => mt2ep,
      mt_rmq_o  => ep2mt,

      eth_src_i => wrc_snk_out,
      eth_src_o => wrc_snk_in,
      eth_snk_i => wrc_src_out,
      eth_snk_o => wrc_src_in
      );
  
  U_GPIO_CPU0 : xwb_gpio_port
    generic map (
      g_INTERFACE_MODE         => PIPELINED,
      g_ADDRESS_GRANULARITY    => BYTE,
      g_NUM_PINS               => 24,
      g_WITH_BUILTIN_SYNC      => false,
      g_WITH_BUILTIN_TRISTATES => false)
    port map (
      clk_sys_i  => clk_sys,
      rst_n_i    => rst_n_sys,
      slave_i    => dp_wb_out(0),
      slave_o    => dp_wb_in(0),
      gpio_out_o => cpu0_gpio_out,
      gpio_in_i  => cpu_gpio_in,
      gpio_oen_o => cpu0_gpio_oen);

  U_GPIO_CPU1 : xwb_gpio_port
    generic map (
      g_INTERFACE_MODE         => PIPELINED,
      g_ADDRESS_GRANULARITY    => BYTE,
      g_NUM_PINS               => 24,
      g_WITH_BUILTIN_SYNC      => false,
      g_WITH_BUILTIN_TRISTATES => false)
    port map (
      clk_sys_i  => clk_sys,
      rst_n_i    => rst_n_sys,
      slave_i    => dp_wb_out(1),
      slave_o    => dp_wb_in(1),
      gpio_out_o => cpu1_gpio_out,
      gpio_in_i  => cpu_gpio_in,
      gpio_oen_o => cpu1_gpio_oen);

  cpu_gpio_out <= cpu0_gpio_out or cpu1_gpio_out;
  cpu_gpio_oen <= cpu0_gpio_oen or cpu1_gpio_oen;

  -- FP GPIO directions
  fp_gpio1_a2b_o  <= cpu_gpio_oen(0);
  fp_gpio2_a2b_o  <= cpu_gpio_oen(1);
  fp_gpio34_a2b_o <= cpu_gpio_oen(2);

  -- FP GPIO bidir in/out (3 and 4 share the same direction line)
  fp_gpio_gen : for i in 0 to 3 generate

    fp_gpio_b(i)  <= cpu_gpio_out(i) when cpu_gpio_oen(i) = '1' else 'Z';
    fp_gpio_in(i) <= fp_gpio_b(i);

    gpio_sync_ffs : gc_sync_ffs
      port map (
        clk_i    => clk_sys,
        rst_n_i  => rst_n_sys,
        data_i   => fp_gpio_in(i),
        synced_o => cpu_gpio_in(i));

  end generate fp_gpio_gen;

  cpu_gpio_in(23 downto 4) <= cpu_gpio_out(23 downto 4);

  U_LED_Controller : gc_bicolor_led_ctrl
    generic map(
      g_NB_COLUMN    => 4,
      g_NB_LINE      => 2,
      g_CLK_FREQ     => 62500000,       -- in Hz
      g_REFRESH_RATE => 250)            -- in Hz
    port map(
      rst_n_i         => rst_n_sys,
      clk_i           => clk_sys,
      led_intensity_i => "1100100",     -- in %
      led_state_i     => cpu_gpio_out(23 downto 8),
      column_o        => fp_led_column_o,
      line_o          => fp_led_line_o,
      line_oen_o      => fp_led_line_oen_o);


  -----------------------------------------------------------------------------
  -- The WR PTP core board package (WB Slave + WB Master #2 (Etherbone))
  -----------------------------------------------------------------------------

  U_xwrc_board_svec : xwrc_board_svec
    generic map (
      g_simulation                => f_to_integer(g_simulation),
      g_with_external_clock_input => false,
      g_dpram_initf               => "../../ip_cores/wr-cores/bin/wrpc/wrc_phy8.bram",
      g_fabric_iface              => PLAIN)
    port map (
      clk_20m_vcxo_i      => clk_20m_vcxo_i,
      clk_125m_pllref_p_i => clk_125m_pllref_p_i,
      clk_125m_pllref_n_i => clk_125m_pllref_n_i,
      clk_125m_gtp_n_i    => clk_125m_gtp_n_i,
      clk_125m_gtp_p_i    => clk_125m_gtp_p_i,
      clk_10m_ext_i       => '0',
      areset_n_i          => areset_n,
      clk_sys_62m5_o      => clk_sys,
      clk_ref_125m_o      => open,
      rst_sys_62m5_n_o    => rst_n_sys,
      pll20dac_din_o      => pll20dac_din_o,
      pll20dac_sclk_o     => pll20dac_sclk_o,
      pll20dac_sync_n_o   => pll20dac_sync_n_o,
      pll25dac_din_o      => pll25dac_din_o,
      pll25dac_sclk_o     => pll25dac_sclk_o,
      pll25dac_sync_n_o   => pll25dac_sync_n_o,
      sfp_txp_o           => sfp_txp_o,
      sfp_txn_o           => sfp_txn_o,
      sfp_rxp_i           => sfp_rxp_i,
      sfp_rxn_i           => sfp_rxn_i,
      sfp_det_i           => sfp_mod_def0_i,
      sfp_sda_i           => sfp_sda_in,
      sfp_sda_o           => sfp_sda_out,
      sfp_scl_i           => sfp_scl_in,
      sfp_scl_o           => sfp_scl_out,
      sfp_rate_select_o   => sfp_rate_select_o,
      sfp_tx_fault_i      => sfp_tx_fault_i,
      sfp_tx_disable_o    => sfp_tx_disable_o,
      sfp_los_i           => sfp_los_i,
      eeprom_sda_i        => eeprom_sda_in,
      eeprom_sda_o        => eeprom_sda_out,
      eeprom_scl_i        => eeprom_scl_in,
      eeprom_scl_o        => eeprom_scl_out,
      onewire_i           => onewire_data,
      onewire_oen_o       => onewire_oe,
      uart_rxd_i          => uart_rxd_i,
      uart_txd_o          => uart_txd_o,
      spi_sclk_o          => spi_sclk_o,
      spi_ncs_o           => spi_ncs_o,
      spi_mosi_o          => spi_mosi_o,
      spi_miso_i          => spi_miso_i,
      wb_slave_o          => cnx_master_in(c_SLAVE_WR_CORE),
      wb_slave_i          => cnx_master_out(c_SLAVE_WR_CORE),

      wrf_src_o => wrc_src_out,
      wrf_src_i => wrc_src_in,
      wrf_snk_o => wrc_snk_out,
      wrf_snk_i => wrc_snk_in
      );

  -- tri-state Carrier EEPROM
  carrier_sda_b <= '0' when (eeprom_sda_out = '0') else 'Z';
  eeprom_sda_in <= carrier_sda_b;
  carrier_scl_b <= '0' when (eeprom_scl_out = '0') else 'Z';
  eeprom_scl_in <= carrier_scl_b;

  -- Tristates for SFP EEPROM
  sfp_mod_def1_b <= '0' when sfp_scl_out = '0' else 'Z';
  sfp_mod_def2_b <= '0' when sfp_sda_out = '0' else 'Z';
  sfp_scl_in     <= sfp_mod_def1_b;
  sfp_sda_in     <= sfp_mod_def2_b;

  -- tri-state onewire access
  onewire_b    <= '0' when (onewire_oe = '1') else 'Z';
  onewire_data <= onewire_b;


end architecture arch;
